# flags: --no-preview-line-length-1
# l2 loses the comment with line-length=1 in preview mode
l1 = ["This list should be broken up", "into multiple lines", "because it is way too long"]
l2 = ["But this list shouldn't", "even though it also has", "way too many characters in it"]  # fmt: skip
l3 = ["I have", "trailing comma", "so I should be braked",]

# output

# l2 loses the comment with line-length=1 in preview mode
l1 = [
    "This list should be broken up",
    "into multiple lines",
    "because it is way too long",
]
l2 = ["But this list shouldn't", "even though it also has", "way too many characters in it"]  # fmt: skip
l3 = [
    "I have",
    "trailing comma",
    "so I should be braked",
]
